/** 
 * Copyright (C) 2008 Alan Ross, ( http://lab.adjazent.com )
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
package ar.particlepainter.paint
{
	import ar.particlepainter.particles.DistortParticle;
	import ar.particlepainter.particles.Particle;
	
	import flash.display.BitmapData;	

	/**
	 * @author Alan Ross
	 */
	public class DistortSpikePaint implements IPaint
	{
		private const NUM_PARTICLES: int = 1000;
		
		private const PI2: Number = Math.PI * 2;
		
		private var _canvas: BitmapData;
		
		private var _radius: int;
		
		private var _particles: DistortParticle;
	
		/**
		 * Create a new <code>DistortSpikePaint</code> object.
		 */
		public function DistortSpikePaint( canvas: BitmapData, radius: int = 20 )
		{
			_canvas = canvas;
			
			_radius = radius;
			
			var p: DistortParticle;
			
			p = _particles = new DistortParticle();
			
			var n: int = NUM_PARTICLES;
			
			while( --n > -1 )
			{
				p = p.next = new DistortParticle();
			}
		}
		
		/**
		 * @inheritDoc
		 */
		public function paint( mouseX: int, mouseY: int, mouseDown: Boolean ): void
		{
			var temp: Number;
			
			var p: DistortParticle = _particles.next;
			
			do
			{	
				--p.restLife;
				
				if( p.restLife < 0 )
				{
					if( mouseDown )
					{
						temp = Math.random() * PI2;
						
						p.distToOrigin = ( _radius * Math.random() ) / p.size;
						
						p.x = mouseX + ( Math.sin( temp * PI2 ) * p.distToOrigin );
						p.y = mouseY + ( Math.cos( temp * PI2 ) * p.distToOrigin );
						
						p.color = _canvas.getPixel32( p.x, p.y );
						
						p.dirX = Math.sin( temp * PI2 );
						p.dirY = Math.cos( temp * PI2 );
					}
					else
					{
						p.x = -1;
						p.y = -1;
					}	
					
					p.restLife = p.lifespan;
				}
				else
				{
					if( p.x != -1 && p.y != -1 )
					{
						//p.dirY *= Math.random() - Math.random();
						//p.dirY *= Math.random() - Math.random();
						p.x += p.dirX * Math.random() - Math.random();
						p.y += p.dirY * Math.random() - Math.random();
						
						p.paint( _canvas );
					}
				}
				
				p = p.next;
			} 
			while( p );
		}
	}
}
