/** 
 * Copyright (C) 2008 Alan Ross, ( http://lab.adjazent.com )
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
package ar.particlepainter.paint
{
	import ar.particlepainter.particles.NoiseSandParticle;
	
	import flash.display.BitmapData;
	import flash.filters.BlurFilter;
	import flash.geom.Rectangle;	

	/**
	 * @author Alan Ross
	 */
	public class NoiseSandPaint implements IPaint
	{
		private const NUM_PARTICLES: int = 5000;
		
		private const PI2: Number = Math.PI * 2;
		
		private var _canvas: BitmapData;
		
		private var _radius: int;
		
		private var _particles: NoiseSandParticle;
		
		private var _blur: BlurFilter;

		/**
		 * Create a new <code>NoiseSandPaint</code> object.
		 */
		public function NoiseSandPaint( canvas: BitmapData, radius: int = 1, color: uint = 0xff0099 )
		{
			_canvas = canvas;
			_radius = radius;
			
			_blur = new BlurFilter( 1, 1, 3 );
			
			var p: NoiseSandParticle;
			
			p = _particles = new NoiseSandParticle( );
			
			var n: int = NUM_PARTICLES;
			
			while( --n > -1 )
			{
				p = p.next = new NoiseSandParticle();
				p.color = color * Math.random();
			}
		}
		
		/**
		 * @inheritDoc
		 */
		public function paint( mouseX: int, mouseY: int, mouseDown: Boolean ): void
		{
			var temp: Number;
			var p: NoiseSandParticle = _particles.next;
			
			do
			{	
				--p.restLife;
				
				if( p.restLife < 0 )
				{
					if( mouseDown )
					{
						p.distToOrigin = ( _radius * Math.random() ) / p.size;
						
						temp = Math.random() * PI2;
						
						p.x = mouseX + ( Math.sin( temp ) * p.distToOrigin );
						p.y = mouseY + ( Math.cos( temp ) * p.distToOrigin );
						
						temp = Math.random() * PI2;
						
						p.dirX = Math.sin( temp );
						p.dirY = Math.cos( temp );
					}
					else
					{
						p.x = -1;
						p.y = -1;
					}	
					
					p.restLife = p.lifespan;
				}
				else
				{
					if( p.x != -1 && p.y != -1 )
					{
						p.x += ( p.dirX * ( p.size * Math.random() ) );
						p.y += ( p.dirY * ( p.size * Math.random() ) );
						
						p.alpha = 0xFF * ( 1 - ( p.distToOrigin / _radius ) );
					
						p.paint( _canvas );
					}
				}
				
				p = p.next;
			} 
			while( p );
			
			_canvas.applyFilter( _canvas, _canvas.rect, _canvas.rect.topLeft, _blur );
		}
	}
}

