/** 
 * Copyright (C) 2008 Alan Ross, ( http://lab.adjazent.com )
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
package ar.particlepainter.particles 
{
	import ar.particlepainter.particles.Particle;
	
	import flash.display.BitmapData;	

	/**
	 * @author Alan Ross
	 */
	public class NoiseSandParticle extends Particle
	{
		private const PI2: Number = Math.PI * 2;
		
		public var next: NoiseSandParticle;
		
		public var viscosity: Number;
		
		public var distToOrigin: int;
		
		public var size: int;
		
		
		/**
		 * Create a new <code>NoiseSandParticle</code> object.
		 */
		public function NoiseSandParticle()
		{
			var maxDroplength: int = 8;
			var minDroplength: int = 5;
			
			if( Math.random() > 0.9 )
				lifespan = 1 + ( maxDroplength * Math.random() );
			else
				lifespan = 1 + ( ( minDroplength * 0.5 ) * Math.random() );
				
			size = Math.random() * 9;
			
			viscosity = 0.1;
		}

		/**
		 * @inheritDoc
		 */
		override public function paint( b: BitmapData ): void
		{
			color32 = ( ( 0xFF << 24 ) | color );
			
			var arcRand: Number;
			var px: int;
			var py: int;
			var n: int = 1;
			var temp: Number = size * 0.006;
			
			while( --n > -1 )
			{
				arcRand = Math.random() * PI2;
				
				px = x + ( Math.sin( arcRand ) * temp );
				py = y + ( Math.cos( arcRand ) * temp );
				
				b.setPixel32( px, py, color32 );
			}
		}
	}
}
